"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dom_1 = require("./dom");
const Highlight_1 = require("./Highlight");
const injectHighlightWrappers_1 = require("./injectHighlightWrappers");
const rangeContents_1 = require("./rangeContents");
const removeHighlightWrappers_1 = require("./removeHighlightWrappers");
const selection_1 = require("./selection");
const SerializedHighlight_1 = require("./SerializedHighlight");
class Highlighter {
    constructor(container, options = {}) {
        this.highlights = {};
        this.eraseAll = () => {
            this.getHighlights().forEach(this.erase);
        };
        this.erase = (highlight) => {
            removeHighlightWrappers_1.default(highlight);
            delete this.highlights[highlight.id];
        };
        this.onMouseup = (ev) => {
            const selection = this.document.getSelection();
            if (!selection) {
                return;
            }
            if (selection.isCollapsed) {
                this.onClick(ev.target);
            }
            else {
                this.onSelect(selection);
            }
        };
        this.container = container;
        this.options = Object.assign({ className: 'highlight' }, options);
        this.container.addEventListener('mouseup', this.onMouseup);
    }
    unmount() {
        this.container.removeEventListener('mouseup', this.onMouseup);
    }
    highlight(highlight) {
        if (highlight instanceof SerializedHighlight_1.default && highlight.isLoadable(this)) {
            return this.highlight(highlight.load(this));
        }
        else if (highlight instanceof Highlight_1.default) {
            this.highlights[highlight.id] = highlight;
            injectHighlightWrappers_1.default(highlight, this.options);
        }
    }
    getHighlight(id) {
        return this.highlights[id];
    }
    getReferenceElement(id) {
        return this.container.querySelector(`[id="${id}"]`);
    }
    clearFocus() {
        this.container.querySelectorAll(`.${this.options.className}.${Highlight_1.FOCUS_CSS}`)
            .forEach((el) => el.classList.remove(Highlight_1.FOCUS_CSS));
    }
    getHighlights() {
        return Object.values(this.highlights);
    }
    getOrderedHighlights() {
        const highlights = Object.values(this.highlights);
        highlights.sort((a, b) => {
            return a.range.compareBoundaryPoints(Range.START_TO_START, b.range);
        });
        return highlights;
    }
    getHighlightBefore(target) {
        return this.getOrderedHighlights().filter((highlight) => highlight.id !== target.id &&
            highlight.range.compareBoundaryPoints(Range.START_TO_START, target.range) < 0).pop();
    }
    getHighlightAfter(target) {
        return this.getOrderedHighlights().filter((highlight) => highlight.id !== target.id &&
            highlight.range.compareBoundaryPoints(Range.START_TO_START, target.range) >= 0).shift();
    }
    get document() {
        if (!this.container.ownerDocument) {
            throw new Error('highlighter container is not mounted to a document!');
        }
        return this.container.ownerDocument;
    }
    onClick(target) {
        const { onClick } = this.options;
        if (!onClick) {
            return;
        }
        if (dom_1.default(target).isHtmlElement) {
            target = dom_1.default(target);
            while (target.isHtmlElement) {
                if (target.el.getAttribute(injectHighlightWrappers_1.DATA_ATTR)) {
                    // there may be multiple highlighters active on the same document,
                    // check if the found highlight is known to this instance
                    const highlight = this.highlights[target.el.getAttribute(injectHighlightWrappers_1.DATA_ID_ATTR)];
                    if (highlight) {
                        onClick(highlight);
                        return;
                    }
                }
                target = dom_1.default(target.el.parentElement);
            }
        }
        onClick();
    }
    onSelect(selection) {
        const { onSelect } = this.options;
        const range = selection_1.snapSelection(selection, this.options);
        if (onSelect && range) {
            const highlights = Object.values(this.highlights)
                .filter((other) => other.intersects(range));
            if (highlights.length === 0) {
                const highlight = new Highlight_1.default(range, { content: rangeContents_1.rangeContentsString(range) });
                onSelect(highlights, highlight);
            }
            else {
                onSelect(highlights);
            }
        }
    }
}
exports.default = Highlighter;
//# sourceMappingURL=Highlighter.js.map